--
-- Very simple addon that adds a note to NPC tooltips when available.
-- If you want to add your own notes in-game, simply do
--  /script MobNotesDB["Name of mob"] = "Note"
-- (note that %t doesn't work with /script)
--
-- Database upgrades (i.e. if the addon is updated on the SVN with new notes)
-- will NOT overwrite or change any custom notes you have added.
--

--[[
<Pkekyo|> much easier to ninja out of combatlog what CAN'T be done imo - for the end user, atleast
<vhaarr> I'm not interested in your opinion
<ag`> :p
<kergoth> write a little addon to ninja those messages and inject them into mobnotes's SV, perhaps.  then it's still flexible for adding any notes, but would gain those bits automatically?
<vhaarr> kergoth: that could be interesting, at least if those "Your Polymorph fails. X is immune." and "Your Wound Poison fails. X is immune." etc messages are generated from the same event.
<vhaarr> kergoth: actually it's CHAT_MSG_SPELL_SELF_DAMAGE
<vhaarr> which is fired pretty often

-- Could still be interesting to fool around with it as a separate addon, but I
-- don't think I want it included here. Perhaps wait for 2.4 and see then.
--
--]]

-- Obviously bump this number if you add any new notes. Note that changing old
-- notes will not upgrade the users DB.
local DB_VERSION = 5

-- The default database should only include strategy-neutral comments, like what
-- mobs are capable of doing and how they can be crowd controlled, or if they
-- are immune to everything, like sheeps or stuns. Don't include any strategy
-- like "kill this mob first" or "have your offtank on this one".
local defaultDatabase = nil

local POPUP_TEXT = "Set a note for |cffff4488%s|r."
local TARGET_ERROR = "You must target something to add a note."
local NOTE_SET = "Note for |cffff4488%s|r set to \"|cff888888%s|r\"."
local SET = "Set"
local CANCEL = "Cancel"
local NOTE_PREFIX = "|cffff4488Note:|r "

--local l = GetLocale()
-- if l == "enUS" then
defaultDatabase = {
	-- Zul'Aman trash
	["Amani'shi Guardian"] = "Immune to snares, polymorph, and fear. Dispels sheep from nearby friends.",
	["Amani'shi Medicine Man"] = "Can be sheeped, snared and MC'ed. Nuke down his totems.",
	["Amani'shi Tribesman"] = "Can be CC'ed by all means.",
	["Amani Bear"] = "Can use Tranquilizing Shot on their frenzy. Freezing Trap, Polymorph and Fear works.",
	["Amani'shi Warbringer"] = "Immune to CC. Dismounts around 20%.",
	["Amani'shi Scout"] = "Can be rooted and stunlocked, prevent them from reaching the drums.",
	["Amani'shi Handler"] = "Immune to Polymorph and Fear, can be stunned. Uses Tranquilizing Poison.",
	["Amani'shi Tempest"] = "Engaging him stops the waves. Does a nasty AoE that can be outranged.",
	["Amani'shi Flame Caster"] = "Vulnerable to Polymorph, Mind Control, snares, stuns and interrupts. Can spell steal his buff.",
	["Amani'shi Beast Tamer"] = "Can be sheeped, trapped and stunned. Mind Controls random players.",

	-- Moroes adds
	-- ZZZ: It seems people can't agree on the name of Catriona and Rafe, so we
	-- just include both their names.
	["Lady Catriona Von'Indi"] = "Holy priest, heals and dispels.",
	["Lady Catriona Von'linda"] = "Holy priest, heals and dispels.",
	["Baron Rafe Dreuger"] = "Retribution paladin, will stun.",
	["Baron Rafe Drueger"] = "Retribution paladin, will stun.",
	["Lady Keira Berrybuck"] = "Holy paladin, heals and dispels + divine shield.",
	["Baroness Dorothea Millstipe"] = "Shadow priest with mana burn.",
	["Lord Robin Daris"] = "Mortal Strike warrior, also does WW.",
	["Lord Crispin Ference"] = "Protection warrior, disarm, shield bash and shield wall.",

	-- Hex Lord Malacrass adds
	["Thurg"] = "Melee. Can sheep or trap, but he will resist trap a few times.",
	["Gazakroth"] = "Spams fireballs, can be banished.",
	["Lord Raadan"] = "Frontal Flamebreath & Thunderclap, can be sleeped.",
	["Darkheart"] = "Melee and instant cast AoE fear, can be shackled.",
	["Alyson Antille"] = "Heals can be interrupted and she can be sheeped.",
	["Slither"] = "Poison bolts. Can be sleeped.",
	["Fenstalker"] = "Casts Volatile Infection on players, dealing damage to nearby allies. Can be banished.",
	["Koragg"] = "Can be shackled or feared using Turn Undead.",

	-- Serpentshrine Cavern trash
	["Coilfang Beast-Tamer"] = "Enrages the sporebats, dispelling CC on them. Frontal cleave.",
	["Coilfang Hate-Screamer"] = "AoE silence, 30 yard range.",
	["Serpentshrine Sporebat"] = "Random charge with a minimum range requirement.",

	["Vashj'ir Honor Guard"] = "Intimidating shout and enrage.",
	["Coilfang Shatterer"] = "Warriors can Spell Reflect the Shatter Armor ability.",
	["Coilfang Priestess"] = "Can be CC'ed and stunned by most means.",

	["Greyheart Tidecaller"] = "Can be sheeped and feared. Drops Water Elemental Totem and uses Poison Shield.",
	["Coilfang Serpentguard"] = "Can't be CC'ed, uses frontal Cleave and armor reduction aura.",
	["Greyheart Skulker"] = "Can be sheeped, feared and disarmed.",
	["Greyheart Nether-Mage"] = "Can be sheeped, does AoE damage to the raid.",

	["Coilfang Fathom-Witch"] = "Immune to CC, does a AoE shadow nova with knockback and random mind controls.",

	["Tidewalker Depth-Seer"] = "Drops a HoT on friendlies, can not be CC'ed, but spell interrupts work. Uses Tranquility on low health.",
	["Tidewalker Harpooner"] = "Nets the tank and clears threat, immune to CC.",
	["Tidewalker Hydromancer"] = "Can be stunned, silenced and interrupted. Vulnerable to CoT. Frostbolts and Frost Nova.",
	["Tidewalker Shaman"] = "Immune to all CC, spells can be interrupted.",
	["Tidewalker Warrior"] = "Can be sheeped, feared, snared and disarmed. Random threat dumps.",

	["Greyheart Shield-Bearer"] = "Can be sheeped, sapped and feared.",
	["Serpentshrine Lurker"] = "Can be banished, drops mushrooms.",

	-- Black Temple trash
	["Aqueous Lord"] = "Summons 2 Aqueous Spawns.",
	["Aqueous Spawn"] = "Can be banished and stunned, heals Lords.",
	["Coilskar Sea-Caller"] = "Can be sheeped, casts Hurricane and Forked Lightning, based on LoS.",
	["Coilskar Wrangler"] = "Immune to CC. Frenzies Leviathans.",
	["Leviathan"] = "Immune to CC. Tail Swipe and Poison Spit.",
	["Coilskar Soothsayer"] = "Can be CC'ed, uses Holy Nova.",
	["Coilskar Harpooner"] = "Can be CC'ed, nets people.",
	["Dragon Turtle"] = "Can be CC'ed.",
	["Coilskar General"] = "Can dispel CC on other mobs, possibly based on LoS/range. Immune to CC.",

	-- Tempest Keep trash
	["Apprentice Star Scryer"] = "Can be crowd controlled, does AoE Starshards.",
	["Astromancer"] = "Can be feared, sheeped, trapped and snared. Purge the buff. Does Fireball Volley.",
	["Star Scryer"] = "Can be sheeped, trapped and snared, does AoE Starshards.",
	["Bloodwarder Legionnaire"] = "Cleave and Whirlwind, can only be snared. Immune to fear and trap.",
	["Tempest-Smith"] = "Can be sapped, sheeped and mind controlled. Throws lots of bombs and buffs the Crystalcores.",
	["Bloodwarder Vindicator"] = "Cleanses CC on other mobs, uses Hammer of Justice and Holy Light. Immune to CC.",
	["Bloodwarder Marshal"] = "Whirlwind, immune to CC.",
	["Bloodwarder Squire"] = "Heals, immune to CC and stuns but not counterspells.",
	["Astromancer Lord"] = "Mind controls a random player, immune to CC and stuns.",
	["Tempest Falconer"] = "Fire Shield buff can't be removed. Immune to CC.",
	["Phoenix-Hawk"] = "Charges the player furthest away from the tank. AoE mana burn and knockbacks.",
	["Crystalcore Devastator"] = "Melee knockback, can counter melee attacks for 2k damage and counters ranged with silence. Immune to stuns.",
	["Crystalcore Sentinel"] = "Uninterruptable explosion for 6k and reflectable Overcharge on tank for 16k. Immune to stuns.",
	["Crystalcore Mechanic"] = "Heals Devastators and Sentinels. Random saw blades on raid. Can be banished.",
	["Nether Scryer"] = "Mass mind control.",
	["Crimson Hand Centurion"] = "Whirlwind. Can be sheeped.",
	["Crimson Hand Blood Knight"] = "Stuns tank and dispels CC on mobs, immune to CC.",
	["Crimson Battle Mage"] = "Uses AoE spells and hits for 18k on tanks. Can be feared.",
	["Crimson Hand Inquisitor"] = "Can be crowd controlled, casts Power Infusion.",

	--[""] = "",
}
-- elseif l == "deDE" then
--     defaultDatabase = {}
-- end

local frame = CreateFrame("Frame")
frame:SetScript("OnEvent", function()
	if event == "VARIABLES_LOADED" then
		if not _G.MobNotesDB then
			_G.MobNotesDB = defaultDatabase
			_G.MobNotesDB.VERSION = DB_VERSION
		end

		if not _G.MobNotesDB.VERSION or _G.MobNotesDB.VERSION < DB_VERSION then
			for k, v in pairs(defaultDatabase) do
				if not _G.MobNotesDB[k] then
					_G.MobNotesDB[k] = v
				end
			end
			DEFAULT_CHAT_FRAME:AddMessage("|cffff4488Mob Notes|r database updated to revision " .. tostring(DB_VERSION) .. ".")
			_G.MobNotesDB.VERSION = DB_VERSION
		end

		local orig = GameTooltip:GetScript("OnTooltipSetUnit")
		GameTooltip:SetScript("OnTooltipSetUnit", function(tooltip, ...)
			local name, unitid = tooltip:GetUnit()
			if UnitExists(unitid) and _G.MobNotesDB[name] then
				GameTooltip:AddLine(NOTE_PREFIX .. _G.MobNotesDB[name], 0.5, 0.5, 0.5, 1)
			end
			return orig(tooltip, ...)
		end)
	end
end)

frame:RegisterEvent("VARIABLES_LOADED")

local function addNoteFromPopup(note)
	if type(note) ~= "string" or note:trim():len() == 0 then note = nil end
	local t = UnitExists("target") and UnitName("target") or nil
	if t then
		_G.MobNotesDB[t] = note
		DEFAULT_CHAT_FRAME:AddMessage(NOTE_SET:format(t, tostring(note)))
	end
end

_G.SlashCmdList["MOBNOTES_SHORTHAND"] = function(input)
	if not UnitExists("target") then
		DEFAULT_CHAT_FRAME:AddMessage(TARGET_ERROR)
		return
	end
	if type(input) ~= "string" or input:trim():len() == 0 then
		if not StaticPopupDialogs["MobNotes"] then
			StaticPopupDialogs["MobNotes"] = {
				text = nil,
				button1 = SET,
				button2 = CANCEL,
				whileDead = 1,
				hideOnEscape = 1,
				timeout = 0,
				OnShow = function()
					-- We have to do this onshow to reset the previous text
					local t = UnitExists("target") and UnitName("target") or ""
					_G[this:GetName().."EditBox"]:SetText(_G.MobNotesDB[t] or "")
				end,
				OnHide = function()
					_G[this:GetName().."EditBox"]:SetText("")
				end,
				EditBoxOnEnterPressed = function()
					addNoteFromPopup(_G[this:GetParent():GetName().."EditBox"]:GetText())
					this:GetParent():Hide()
				end,
				EditBoxOnEscapePressed = function()
					this:GetParent():Hide()
				end,
				OnAccept = function()
					addNoteFromPopup(_G[this:GetParent():GetName().."EditBox"]:GetText())
				end,
				hasEditBox = 1,
			}
		end
		StaticPopupDialogs["MobNotes"].text = POPUP_TEXT:format(UnitName("target"))
		StaticPopup_Show("MobNotes")
	else
		local t = UnitName("target")
		_G.MobNotesDB[t] = input
		DEFAULT_CHAT_FRAME:AddMessage(NOTE_SET:format(t, input))
	end
end
_G.SLASH_MOBNOTES_SHORTHAND1 = "/mobnotes"

